@@ -1,17 +1,15 @@
 import datetime
-import pandas as pd
 from typing import Mapping, TypeVar
-from numpy import isin
 
-from pydantic import BaseModel
 import pydantic
+from pydantic import BaseModel
 
 LabelType = TypeVar("LabelType", int, str, datetime.datetime)
 
 
 class Interval(BaseModel):
     left: LabelType
-    right: LabelType
+    right: LabelType = None
 
     @classmethod
     def __get_validators__(cls):
@@ -25,13 +23,14 @@ class Interval(BaseModel):
         elif isinstance(v, Mapping):
             left = v.get("left", None)
             right = v.get("right", None)
-        elif hasattr(v, 'left') and hasattr(v, 'right'):
+
+        elif hasattr(v, "left") and hasattr(v, "left"):
             left = v.left
             right = v.right
         else:
             left, right = v, v
 
-        if left > right:
+        if right is not None and left > right:
             left, right = right, left
         return cls(left=left, right=right)
 
@@ -44,9 +43,10 @@ class Interval(BaseModel):
 
 
 class IntegerInterval(Interval):
-    left = pydantic.conint(ge=0, lt=int(2**32-1))
-    right = pydantic.conint(ge=0, lt=int(2**32-1))
+    left = pydantic.conint(ge=0, lt=int(2**32 - 1))
+    right = pydantic.conint(ge=0, lt=int(2**32 - 1))
+
 
 class TimeInterval(Interval):
     left: datetime.datetime
-    right: datetime.datetime
+    right: datetime.datetime = None
