@@ -1,20 +1,23 @@
 """Main module."""
 
+import re
 from datetime import datetime
-from pydantic.typing import NoneType
-import pymongo
+from typing import Any, List, Tuple, Type, Union
+
 import pandas as pd
+from pydantic.typing import NoneType
 
-from typing import Any, Dict, List, Type, Union, Tuple
-
-from rframe.indexers import get_indexer
+from .interfaces import get_interface
 from .schema import BaseSchema, InsertionError
-from .utils import singledispatchmethod
-from .indexers import get_indexer
 
 IndexLabel = Union[int, float, datetime, str, slice, NoneType, List]
 
 
+def camel_to_snake(name):
+    name = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
+    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", name).lower()
+
+
 class RemoteFrame:
     """Implement basic indexing features similar to a pandas dataframe
     but operates on an arbitrary storage backend
@@ -23,19 +26,7 @@ class RemoteFrame:
     schema: Type[BaseSchema]
     db: Any
 
-    def __init__(self, schema: Type[BaseSchema], db: Any, **kwargs) -> None:
-        if isinstance(db, str):
-            if db.startswith("mongodb"):
-                dbname = kwargs.pop("dbname", "remote_dataframes")
-                db = pymongo.MongoClient(db, **kwargs)[dbname]
-            elif db.endswith(".csv"):
-                db = pd.read_csv(db)
-            elif db.endswith(".pkl"):
-                db = pd.read_pickle(db)
-            elif db.endswith(".pq"):
-                db = pd.read_parquet(db)
-            else:
-                raise TypeError("Unsupported database type")
+    def __init__(self, schema: Type[BaseSchema], db: Any) -> None:
         self.schema = schema
         self.db = db
 
@@ -49,11 +40,13 @@ class RemoteFrame:
 
     @property
     def name(self):
-        return self.schema._name
+        if hasattr(self, "_NAME"):
+            return self._NAME
+        return camel_to_snake(self.schema.__name__)
 
     @property
-    def indexer(self):
-        return get_indexer(self.db)
+    def interface(self):
+        return get_interface(self.db)
 
     @property
     def columns(self):
@@ -117,7 +110,7 @@ class RemoteFrame:
     ) -> Tuple[List[dict], List[dict], List[dict]]:
         """Insert multiple records into the DB"""
         if isinstance(records, pd.DataFrame):
-            records = records.reset_index().to_dict(orient="records")
+            records = self.schema.from_pandas(records)
 
         succeeded = []
         failed = []
@@ -155,7 +148,9 @@ class RemoteFrame:
 
     def __repr__(self) -> str:
         return (
-            f"RemoteDataFrame(" f"index={self.index.names}," f"columns={self.columns})"
+            f"{self.__class__.__name__}("
+            f"index={self.index.names},"
+            f"columns={self.columns})"
         )
 
 
