@@ -1,16 +1,11 @@
-import re
+from typing import Dict, List, Mapping
+
 import pandas as pd
-from typing import Dict, List
 from pydantic import BaseModel
-from pydantic.fields import ModelField, FieldInfo
+from pydantic.fields import FieldInfo, ModelField
 
 from .indexes import BaseIndex, Index, MultiIndex
-from .indexers import get_indexer
-
-
-def camel_to_snake(name):
-    name = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
-    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", name).lower()
+from .interfaces import get_interface
 
 
 class InsertionError(Exception):
@@ -18,25 +13,6 @@ class InsertionError(Exception):
 
 
 class BaseSchema(BaseModel):
-    _name: str = ""
-
-    def __init_subclass__(cls) -> None:
-        if "_name" not in cls.__dict__:
-            cls._name = camel_to_snake(cls.__name__)
-        index_fields = cls.get_index_fields()
-        indexes = []
-        for name, field in index_fields.items():
-            index = field.field_info
-            index.__set_name__(cls, name)
-            indexes.append(index)
-
-        if len(indexes) == 1:
-            index = indexes[0]
-        else:
-            index = MultiIndex(*indexes)
-
-        cls.index = index
-
     @classmethod
     def default_datasource(cls):
         raise NotImplementedError
@@ -85,14 +61,13 @@ class BaseSchema(BaseModel):
         field_info.__set_name__(cls, name)
         return field_info
 
-    @property
-    def index_labels(self):
-        data = self.dict()
-        return {k: data[k] for k in self.get_index_fields()}
+    @classmethod
+    def rframe(cls, datasource=None):
+        import rframe
 
-    @property
-    def index_labels_tuple(self):
-        return tuple(v for v in self.index_labels.values())
+        if datasource is None:
+            datasource = cls.default_datasource()
+        return rframe.RemoteFrame(cls, datasource)
 
     @classmethod
     def compile_query(cls, datastore, **labels) -> List["BaseSchema"]:
@@ -106,9 +81,9 @@ class BaseSchema(BaseModel):
 
         label = index.validate_label(label)
 
-        indexer = get_indexer(datastore)
+        interface = get_interface(datastore)
 
-        return indexer.compile_query(index, label)
+        return interface.compile_query(index, label)
 
     @classmethod
     def _find(cls, datastore, **labels) -> List["BaseSchema"]:
@@ -125,9 +100,9 @@ class BaseSchema(BaseModel):
             label = labels
 
         label = index.validate_label(label)
-        indexer = get_indexer(datastore)
+        interface = get_interface(datastore)
 
-        query = indexer.compile_query(index, label)
+        query = interface.compile_query(index, label)
         docs = query.apply(datastore)
         docs = index.reduce(docs, labels)
         return docs
@@ -150,20 +125,49 @@ class BaseSchema(BaseModel):
         if docs:
             return docs[0]
 
+    @classmethod
+    def from_pandas(cls, record):
+        if isinstance(record, list):
+            return [cls.from_pandas(d) for d in record]
+        if isinstance(record, pd.DataFrame):
+            return [cls.from_pandas(d) for d in record.to_dict(orient="records")]
+
+        if not isinstance(record, Mapping):
+            raise TypeError(
+                "Record must be of type Mapping,"
+                "List[Mapping] or DataFrame],"
+                f"got {type(record)}"
+            )
+        data = dict(record)
+        for name in cls.get_index_fields():
+            index = cls.index_for(name)
+            label = record.get(name, None)
+            data[name] = index.to_pandas(label)
+        return cls(**data)
+
     @classmethod
     def ensure_index(cls, datastore):
-        indexer = get_indexer(datastore)
+        interface = get_interface(datastore)
         names = list(cls.get_index_fields())
-        return indexer.ensure_index(datastore, names)
+        return interface.ensure_index(datastore, names)
+
+    @property
+    def index_labels(self):
+        data = self.dict()
+        return {k: data[k] for k in self.get_index_fields()}
+
+    @property
+    def index_labels_tuple(self):
+        return tuple(v for v in self.index_labels.values())
 
     def save(self, datastore):
-        indexer = get_indexer(datastore)
+        interface = get_interface(datastore)
         existing = self.find(datastore, **self.index_labels)
         if existing:
             existing[0].pre_update(datastore, self)
         else:
             self.pre_insert(datastore)
-        return indexer.insert(datastore, self)
+        return interface.insert(datastore, self)
 
     def pre_insert(self, datastore):
         pass
